import os
import shutil
import warnings
from collections import namedtuple

import numpy as np


def _str(f, size, _):
    s = f.read(size).decode("ascii", "ignore").strip()
    while s.endswith("\x00"):
        s = s[:-1]
    return s


def _int(f, size, name):
    s = _str(f, size, name)
    try:
        return int(s)
    except ValueError:
        warnings.warn("{name}: Could not parse integer {s}.".format(name=name, s=s))


def _float(f, size, name):
    s = _str(f, size, name)
    try:
        return float(s)
    except ValueError:
        warnings.warn("{name}: Could not parse float {s}.".format(name=name, s=s))


def _discard(f, size, _):
    f.read(size)


HEADER = (
    ("version", 8, _str),
    ("local_patient_identification", 80, _str),
    ("local_recording_identification", 80, _str),
    ("startdate_of_recording", 8, _str),
    ("starttime_of_recording", 8, _str),
    ("number_of_bytes_in_header_record", 8, _int),
    ("reserved", 44, _discard),
    ("number_of_data_records", 8, _int),
    ("duration_of_a_data_record", 8, _float),
    ("number_of_signals", 4, _int),
)


SIGNAL_HEADER = (
    ("label", 16, _str),
    ("transducer_type", 80, _str),
    ("physical_dimension", 8, _str),
    ("physical_minimum", 8, _float),
    ("physical_maximum", 8, _float),
    ("digital_minimum", 8, _int),
    ("digital_maximum", 8, _int),
    ("prefiltering", 80, _str),
    ("nr_of_samples_in_each_data_record", 8, _int),
    ("reserved", 32, _discard),
)

INT_SIZE = 2
HEADER_SIZE = sum([size for _, size, _ in HEADER])
SIGNAL_HEADER_SIZE = sum([size for _, size, _ in SIGNAL_HEADER])

Header = namedtuple("Header", [name for name, _, _ in HEADER] + ["signals"])
SignalHeader = namedtuple("SignalHeader", [name for name, _, _ in SIGNAL_HEADER])


def read_edf_header(fd):
    """
    Reads the header of an EDF file.

    Parameters:
        fd (str or file-like object): The file descriptor or the path to the EDF file.

    Returns:
        Header: The header of the EDF file, including information about the signals.

    Raises:
        FileNotFoundError: If the file specified by `fd` does not exist.
        ValueError: If `fd` is not a valid EDF file.
    """
    opened = False
    if isinstance(fd, str):
        opened = True
        fd = open(fd, "rb")

    header = [func(fd, size, name) for name, size, func in HEADER]
    number_of_signals = header[-1]
    signal_headers = [[] for _ in range(number_of_signals)]

    for name, size, func in SIGNAL_HEADER:
        for signal_header in signal_headers:
            signal_header.append(func(fd, size, name))

    header.append(
        tuple(SignalHeader(*signal_header) for signal_header in signal_headers)
    )

    if opened:
        fd.close()

    return Header(*header)


def read_edf_data(fd, header, chans="all"):
    """
    Reads EDF data from a file or file-like object.

    Parameters:
        fd (str or file-like object): The file path or file-like object to read the EDF data from.
        header (Header): The EDF header object containing information about the data.

    Returns:
        generator: A generator that yields each data record as a list of signals.

    Raises:
        IOError: If there was an error opening or reading the EDF file.
    """
    opened = False
    if isinstance(fd, str):
        opened = True
        fd = open(fd, "rb")

        start = 0
        end = header.number_of_data_records

        if chans == "all":
            chans = list(range(header.number_of_signals))

        data_record_length = sum(
            [signal.nr_of_samples_in_each_data_record for signal in header.signals]
        )

        if opened:
            fd.seek(
                HEADER_SIZE
                + header.number_of_signals * SIGNAL_HEADER_SIZE
                + start * data_record_length * INT_SIZE
            )

        for _ in range(start, end):
            a = np.fromfile(fd, count=data_record_length, dtype=np.int16)

            offset = 0
            chan_nr = 0
            data_record = []
            for signal in header.signals:
                if chan_nr in chans:
                    data_record.append(
                        a[offset : offset + signal.nr_of_samples_in_each_data_record]
                    )

                offset += signal.nr_of_samples_in_each_data_record
                chan_nr += 1

            yield data_record

    if opened:
        fd.close()


def write_edf_header(fd, header):
    """
    Writes the EDF header to the specified file-like object or file path.

    Args:
        fd (str): (Relative) path to file to write.
        header: The header information to write.

    Raises:
        AssertionError: If the length of the value to write is not equal to the expected size.

    Returns:
        None
    """
    opened = False
    if isinstance(fd, str):
        fd = open(fd, "wb")
        opened = True

        for val, (name, size, _) in zip(header, HEADER):
            if val is None:
                val = b"\x20" * size

            if not isinstance(val, bytes):
                if (
                    name == "startdate_of_recording" or name == "starttime_of_recording"
                ) and not isinstance(val, str):
                    val = "{:02d}.{:02d}.{:02d}".format(val[0], val[1], val[2] % 100)
                val = str(val).encode(encoding="ascii").ljust(size, b"\x20")

            assert len(val) == size
            fd.write(val)

        for vals, (name, size, _) in zip(zip(*header.signals), SIGNAL_HEADER):
            for val in vals:
                if val is None:
                    val = b"\x20" * size

                if not isinstance(val, bytes):
                    val = str(val).encode(encoding="ascii").ljust(size, b"\x20")

                assert len(val) == size
                fd.write(val)

    if opened:
        fd.close()


def write_edf_data(fd, data_records):
    """Function to check and fix edf files according to EDF plus standards

    Args:
        fd (str): (Relative) path to file to write.
        data_records (array): Variable with data_records to write to edf file\
            is output from read_edf_data function

    """
    opened = False
    if isinstance(fd, str):
        opened = True
        fd = open(fd, "ab")
        for data_record in data_records:
            for signal in data_record:
                signal.tofile(fd)
    # try:

    if opened:
        fd.close()


def fix_edf_header(fd):
    """
    Fix the EDF header.

    Args:
        fd (file object): The file descriptor for the EDF file.

    Returns:
        None
    """

    if not (os.path.isfile(fd)):
        raise FileNotFoundError(fd)
    else:
        print(f"fixing header for {fd} ... ", end="", flush=True)

    header = read_edf_header(fd)
    data = read_edf_data(fd, header)

    something_to_fix = False
    if ":" in header.startdate_of_recording:
        warnings.warn(
            f"start date {header.startdate_of_recording} "
            "contains colon (:), changing to dot (.)"
        )
        header = header._replace(
            startdate_of_recording=header.startdate_of_recording.replace(":", ".")
        )
        something_to_fix = True

    if ":" in header.starttime_of_recording:
        warnings.warn(
            f"start time {header.starttime_of_recording}"
            "contains colon (:), changing to dot (.)"
        )
        header = header._replace(
            starttime_of_recording=header.starttime_of_recording.replace(":", ".")
        )
        something_to_fix = True

    for signal in header.signals:
        if signal.physical_maximum <= signal.physical_minimum:
            warnings.warn(
                f"channel {signal.label}: physical maximum "
                f"({signal.physical_maximum}) is smaller or equal "
                f"to physical minimum ({signal.physical_minimum})"
            )

        if signal.digital_maximum <= signal.digital_minimum:
            warnings.warn(
                f"channel {signal.label}: digital maximum "
                f"({signal.digital_maximum}) is smaller or equal "
                f"to digital minimum ({signal.digital_minimum})"
            )

    if something_to_fix:
        tmp_fd = fd + "tmp"
        write_edf_header(tmp_fd, header)
        write_edf_data(tmp_fd, data)

        assert os.path.getsize(tmp_fd) == os.path.getsize(fd)
        os.replace(tmp_fd, fd)

    print("done")


def anonymize_edf_header(fd):
    """Function to anonymize edf files according to ENSEMBLE and BIDS standards.
    The output file will be appended with ANONYMIZED in the filename. Please
    make sure to only upload the anonymized files.


    Args:
    fd (str): (Relative) path to file to rename.

    """
    if not (os.path.isfile(fd)):
        raise FileNotFoundError(fd)
    else:
        print(f"anonymizing {fd} ... ", end="", flush=True)

    header = read_edf_header(fd)
    data = read_edf_data(fd, header)

    filename = os.path.splitext(os.path.basename(fd))[0]
    ext = os.path.splitext(os.path.basename(fd))[1]
    folder = os.path.dirname(fd)
    split_filename = filename.split("_")
    is_ensemble_approved = (
        split_filename[0] == "subj"
        and len(split_filename[1]) == 10
        and ("E" in split_filename[1])
    )

    anonymized_rid = "Startdate X X X X"
    if is_ensemble_approved:
        pseudo_code = split_filename[1]
        anonymized_pid = pseudo_code + " X X X"
    else:
        anonymized_pid = "X X X X"

    anonymized_startdate = "01.01.85"
    anonymized_starttime = "00.00.00"

    header = header._replace(
        local_patient_identification=anonymized_pid,
        local_recording_identification=anonymized_rid,
        startdate_of_recording=anonymized_startdate,
        starttime_of_recording=anonymized_starttime,
    )

    fd_out = os.path.join(folder, filename + "_ANONYMIZED" + ext)
    write_edf_header(fd_out, header)
    write_edf_data(fd_out, data)

    print("done")


def rename_for_ensemble(fd):
    """Function to rename edf files according to ENSEMBLE and BIDS standards

    Args:
        fd (str): (Relative) path to file to rename.

    """
    if not os.path.isfile(fd):
        raise FileNotFoundError(fd)

    filedir = os.path.expanduser(os.path.dirname(fd))
    if not filedir:
        filedir = os.getcwd()

    while True:
        print(f"changing name of {fd}")
        filename = os.path.basename(fd)

        # check whether filename needs to be changed
        do_renaming = check_filename_ensemble(filename)

        if do_renaming:
            header = read_edf_header(fd)

            # get the new subject code
            subject_code = get_subject_code()

            # check type of acquisition
            acq = get_acquisition_type(header)

            # check type of session
            ses = get_session_type()

        split_new_filename = [subject_code, ses, acq, "run-1_eeg.edf"]
        new_filename = "_".join(split_new_filename)

        print(f"new filename is {new_filename}")
        correct_filename = input("Is this correct? [Y/n]: ")

        if correct_filename.lower() == "y":
            break

    # Create new directory and copy renamed file
    new_dirname = os.path.join(filedir, subject_code)
    new_filename = os.path.join(new_dirname, new_filename)

    if not os.path.isdir(new_dirname):
        os.mkdir(new_dirname)

    if os.path.isfile(new_filename):
        print("File already exists, not overwriting")
    else:
        shutil.copy(fd, new_filename)


def combine_aeeg_channels(fd_left, fd_right, new_filename="two_channel_aeeg"):
    """
    Combine left and right aEEG channels into a single edf file.

    Args:
        fd_left (str): The file path of the left aEEG channel.
        fd_right (str): The file path of the right aEEG channel.
        new_filename (str, optional): The name of the new combined file. Defaults to "two_channel_aeeg".

    Raises:
        FileNotFoundError: If fd_left or fd_right is not a valid file path.
    """
    if not os.path.isfile(fd_left):
        raise FileNotFoundError(fd_left)
    elif not os.path.isfile(fd_right):
        raise FileNotFoundError(fd_right)

    filename_left = os.path.basename(fd_left)
    filename_right = os.path.basename(fd_right)
    print(f"Combining {filename_left} and {filename_right} ... ", end="", flush=True)

    output_dir = os.path.dirname(fd_left)
    path_to_file = os.path.join(output_dir, new_filename + ".edf")

    hdr_left = read_edf_header(fd_left)
    hdr_right = read_edf_header(fd_right)

    opened = False
    if isinstance(fd_left, str):
        opened = True
        fd = open(fd_left, "rb")

    # create new header by combining left and right channel headers, get the
    # annotation channel from right channel
    header = [func(fd, size, name) for name, size, func in HEADER]

    # create new signal headers
    signal_headers_left = [hdr_left.signals[0]]
    signal_headers_right = [signal for signal in hdr_right.signals]
    signal_headers = signal_headers_left + signal_headers_right
    header[-1] = len(signal_headers)
    header.append(
        tuple(SignalHeader(*signal_header) for signal_header in signal_headers)
    )

    # calculate new header length
    header_length = HEADER_SIZE + len(signal_headers) * SIGNAL_HEADER_SIZE
    header[5] = header_length

    # read in data from both left and right channels, read - if possible -
    # annotation channel from right channel edf file
    labels_left = [signal.label for signal in hdr_left.signals]
    chans = list(range(hdr_left.number_of_signals))
    if "EDF Annotations" in labels_left:
        chans.remove(labels_left.index("EDF Annotations"))

    data_left = list(read_edf_data(fd_left, hdr_left, chans=chans))
    data_right = list(read_edf_data(fd_right, hdr_right))

    # append channels
    data_all = append_channels(data_left, data_right)

    write_edf_header(path_to_file, Header(*header))
    write_edf_data(path_to_file, data_all)

    if opened:
        fd.close()

    print("done")


def append_channels(data_left, data_right):
    """
    This function takes two lists of data and appends the corresponding elements from the second list to the elements in the first list. It yields the appended elements.
    """
    counter = 0
    for _ in range(len(data_left)):
        dat_out = data_left[counter]
        dat_out.extend(data_right[counter])
        counter += 1

        yield dat_out


def check_filename_ensemble(filename):
    """
    Helper function to check filename and compare to the ENSEMBLE standard

     Args:
        fd (str): (Relative) path to file to rename.

    """
    split_filename = filename.split("_")
    do_renaming = True

    if (
        (split_filename[0] == "subj")
        and ("ses" in split_filename[2])
        and ("acq" in split_filename[3])
        and ("run" in split_filename[4])
        and (split_filename[-1] == "eeg.edf")
    ):
        do_renaming = False
        warnings.warn(
            "The filename already seems to adhere to the ensemble " "and BIDS standard "
        )
        continue_renaming = input("Are you sure you want to rename this file? " "[y/N]")
        do_renaming = continue_renaming.lower() == "y"

    return do_renaming


def get_subject_code():
    """
    Helper code to get subject code with user input
    """

    # Get centre code
    while True:
        centre_code = input("Please input your centre code [xxx]: ")
        if len(centre_code) != 3 or not centre_code.isdigit():
            print("Centre code must consist of three digits")
            continue
        break

    # Get subject number
    while True:
        subject_number = input("Please input your subject number [xxxxx]: ")
        if len(subject_number) != 5 or not subject_number.isdigit():
            print("Subject number must consist of five digits")
            continue
        break

    # Get sibling number
    while True:
        sibling_number = input("Please input the sibling number [x]: ")
        if len(sibling_number) != 1 or not sibling_number.isdigit():
            print("Sibling number must consist of a single digit")
            continue
        break

    subject_code = "sub-" + centre_code + "E" + subject_number + sibling_number

    return subject_code


def get_acquisition_type(header):
    """
    Helper code to get acquisition type with user input or header information.

    Args:
        header: The header information of the file.

    Returns:
        acq: The determined acquisition type.
    """
    # Check number of signals in file
    if header.number_of_signals <= 4:
        acq = "acq-aeeg"
        print("file automatically determined to be aEEG")
        correct_acq = input("is this correct? [Y/n]: ").lower()

        if correct_acq == "n":
            acq = "acq-ceeg"

    else:
        acq = "acq-ceeg"
        print("file automatically determined to be cEEG")
        correct_acq = input("is this correct? [Y/n]: ").lower()

        if correct_acq == "n":
            acq = "acq-aeeg"

    return acq


def get_session_type():
    """
    Helper code to get session type with user input
    """
    while 1:
        ses_string = (
            "During which session was this recordig taken? " + "[(d)iag/(t)erm]: "
        )
        ses = input(ses_string).lower()
        if ses == "d" or ses == "diag":
            ses = "ses-diag"
            break
        elif ses == "t" or ses == "term":
            ses = "ses-term"
            break

    return ses



import glob
import os
import shutil
import zipfile
from collections import Counter, namedtuple
from datetime import datetime, timezone

import defusedxml.ElementTree as ET
import numpy as np

import src.ensemble_eeg.ensemble_eeg as ensemble_eeg


def convert_brm_to_edf(fd, is_fs_64hz=None):
    """
    Converts a BRM file to EDF format.

    Parameters:
        fd (str): The path to the BRM file.
        is_fs_64hz (bool, optional): Indicates whether the sampling frequency is 64 Hz. Defaults to None.

    Raises:
        ValueError: If the file is not found.

    Returns:
        None
    """
    filename = os.path.basename(fd)
    file_exists = os.path.isfile(fd)

    if file_exists:
        file_dir = os.path.dirname(fd)
        tmp_dir = os.path.join(file_dir, "tmp")
        if os.path.isdir(tmp_dir):
            shutil.rmtree(tmp_dir)

        print(f"{filename}")
        print("\tUnzipping to temporary directory")
        with zipfile.ZipFile(fd, "r") as zip_ref:
            zip_ref.extractall(tmp_dir)

        index_xml = os.path.join(tmp_dir, "BRM_Index.xml")
        index = parse_xml(index_xml)
        device_xml = os.path.join(tmp_dir, "Device.xml")
        device = parse_xml(device_xml)

        dat_files_left = []
        dat_files_right = []

        if is_fs_64hz is None:
            is_fs_64hz = (
                input("is the sampling " "frequency 64 Hz? [y/N]: ").lower() == "y"
            )

        if is_fs_64hz:
            dat_files_left = sorted(
                glob.glob(os.path.join(tmp_dir, "DATA_RAW_EEG_LEFT*.dat"))
            )
            dat_files_right = sorted(
                glob.glob(os.path.join(tmp_dir, "DATA_RAW_EEG_RIGHT*.dat"))
            )

        else:
            dat_files_left = sorted(
                glob.glob(os.path.join(tmp_dir, "DATA_RAW_EEG_ELECTRODE_LEFT*.dat"))
            )
            dat_files_right = sorted(
                glob.glob(os.path.join(tmp_dir, "DATA_RAW_EEG_ELECTRODE_RIGHT*.dat"))
            )

        n_data_files_left = len(dat_files_left)
        n_data_files_right = len(dat_files_right)

        assert n_data_files_left == n_data_files_right

        for i in range(0, n_data_files_left):
            both_dat_files = [dat_files_left[i], dat_files_right[i]]
            data = extract_brm_file(index, device, both_dat_files)

            if i == 0:
                output_filename = os.path.splitext(fd)[0] + ".edf"
            else:
                output_filename = os.path.splitext(fd)[0] + "_" + str(i) + ".edf"

            hdr = prepare_edf_header(data)
            signal_header = prepare_edf_signal_header(data, device)
            header = ensemble_eeg.Header(*hdr, signal_header)

            # write header to file
            print(f"\tprint header to {output_filename}")
            ensemble_eeg.write_edf_header(output_filename, header)

            # write data to file
            print(f"\tprint data records to {output_filename}")
            write_brm_data_to_edf(output_filename, data)

        print("\tremoving temporary directory")
        shutil.rmtree(tmp_dir)
    else:
        raise ValueError("file not found")


def parse_xml(xml):
    """
    Parse an XML file and return a named tuple representing the parsed XML.

    Parameters:
        xml (str): The path to the XML file to be parsed.

    Returns:
        Parsed_XML: A named tuple representing the parsed XML. The fields of the
            named tuple correspond to the unique XML tags found in the file. If
            a tag appears only once, the corresponding field of the named tuple
            will contain the text content of the tag. If a tag appears multiple
            times, the corresponding field of the named tuple will contain a
            list of named tuples, each representing one occurrence of the tag.
            The fields of the nested named tuples correspond to the unique tags
            found within the repeated tag, and the values of the fields
            correspond to the text content of the corresponding tags.

    Raises:
        ValueError: If the specified XML file does not exist.

    Example:
        >>> parse_xml('data.xml')
        Parsed_XML(tag_names=['tag1', 'tag2'], tag1='text1', tag2=[Tag2(tag3='text3', tag4='text4')])
    """
    file_exists = os.path.isfile(xml)

    if file_exists:
        tree = ET.parse(xml)
        root = tree.getroot()

        all_tags = [child.tag for child in root]
        tag_names = list(Counter(all_tags).keys())
        tag_dict = list(Counter(all_tags).items())

        Parsed_XML = namedtuple("ParsedXML", tag_names)
        parsed_xml = []
        counter = 0
        for name, vals in tag_dict:
            if vals == 1:
                parsed_xml.append(root[counter].text)
                counter = counter + 1
            else:
                parsed_child = []
                child_elems = root.findall(name)
                for child_elem in child_elems:
                    child_elem_tags = [el.tag for el in child_elem]
                    child_elem_vals = [el.text for el in child_elem]
                    tup = namedtuple(child_elem.tag, child_elem_tags)
                    parsed_child.append(tup(*child_elem_vals))
                    counter = counter + 1
                parsed_xml.append(parsed_child)

    else:
        raise ValueError("file not found")

    return Parsed_XML(*parsed_xml)


def extract_brm_file(index, device, dat_files):
    """
    Extracts data from BRM files based on the given index, device, and dat_files.

    Parameters:
        index (Index): The index object containing file descriptions.
        device (Device): The device to extract data for.
        dat_files (List[str]): The list of dat files to extract data from.

    Returns:
        List[List[Data]]: A list of data extracted from BRM files.
    """
    filenames = [fd.FileName for fd in index.FileDescription]

    counter = 0
    data = [[] for _ in dat_files]

    for dat_file in dat_files:
        df = os.path.basename(dat_file)
        file = index.FileDescription[filenames.index(df)]
        file = file._replace(FileName=dat_file)
        print(f"\textracting {df} datastream")
        data[counter] = get_brm_data(file, device)
        counter = counter + 1

    return data


def get_brm_data(file, device):
    """
    Generates a named tuple containing the data from the given file and device.

    Parameters:
        file (namedtuple): The file containing the data.
        device (str): The device associated with the data.

    Returns:
        Data: A named tuple containing the data from the file, the sample rate, and the numerical data.
    """
    DAUSampleHz = 512

    Data = namedtuple("data", list(file._fields) + ["sampleHz", "data"])
    data = list(file)
    data.append(int(DAUSampleHz / int(file.SamplePeriod512thSeconds)))
    data.append(get_numerical_data(file, device))

    return Data(*data)


def get_numerical_data(file, device):
    """
    Reads numerical data from a file and returns it.

    Parameters:
        file (File): The file object representing the file to read from.
        device (str): The device to use for reading the file.

    Returns:
        np.ndarray: The numerical data read from the file.

    Raises:
        Exception: If the file format is unrecognized.
    """
    f = open(file.FileName, "rb")

    match file.FileType:
        case "FloatMappedToInt16" | "Int16":
            data = np.fromfile(f, dtype=np.int16)
        case "Float32":
            data = np.fromfile(f, dtype=np.float32)
        case _:
            raise Exception(f"Unrecognized file format {file.FileType}")

    return data


def prepare_edf_header(data):
    """
    Generate the header for an EDF (European Data Format) file.

    Args:
        data: A list of objects containing the data for each channel.

    Returns:
        header: A list containing the header information for the EDF file.

    Raises:
        None.

    Examples:
        >>> data = [channel1, channel2]
        >>> prepare_edf_header(data)
        ['0', 'X X X X', 'Startdate X X X X', 'dd.mm.yy', 'hh.mm.ss',
        header_size, None, n_records, '1', n_signals]
    """
    n_channels = 2
    version = "0"
    PID = "X X X X"
    RID = "Startdate X X X X"
    # start_date = datetime.strftime(
    #     datetime.fromtimestamp(int(data[0].StartTime), timezone.utc), format="%d.%m.%y"
    # )
    # start_time = datetime.strftime(
    #     datetime.fromtimestamp(int(data[0].StartTime), timezone.utc), format="%H.%M.%S"
    # )

    start_date = "01.01.85"
    start_time = "00.00.00"

    bytes_in_header = (
        ensemble_eeg.HEADER_SIZE + n_channels * ensemble_eeg.SIGNAL_HEADER_SIZE
    )
    reserved = None
    fs = data[0].sampleHz
    n_records = int(np.floor(len(data[0].data) / fs))
    dur_data_record = "1"
    n_signals = len(data)
    header = [
        version,
        PID,
        RID,
        start_date,
        start_time,
        bytes_in_header,
        reserved,
        n_records,
        dur_data_record,
        n_signals,
    ]

    return header


def prepare_edf_signal_header(data, device):
    """
    Generate the signal headers for an EDF file based on the provided data and device.

    Parameters:
        data (list): A list of data objects.
        device (Device): The device object.

    Returns:
        tuple: A tuple of signal headers.
    """
    channel_ids = [channel.ID for channel in device.Channel]
    units = data[0].Units

    # replace non-ascii characters
    if "µ" in units:
        units = units.replace("µ", "u")

    signal_headers = [[] for _ in data]
    for i in range(0, len(data)):
        ichan = channel_ids.index(data[i].ChannelTitle)
        if data[i].ChannelTitle == "Left":
            label = "F3"
        elif data[i].ChannelTitle == "Right":
            label = "F4"

        transducer_type = None
        physical_dimension = units
        physical_min = round(-5e6 / float(device.Channel[ichan].Gain) / 2)
        physical_max = round(5e6 / float(device.Channel[ichan].Gain) / 2)
        digital_min = -(2**15)
        digital_max = 2**15 - 1
        prefiltering = None
        nr_samples = data[i].sampleHz
        reserved = None
        sig_header = [
            label,
            transducer_type,
            physical_dimension,
            physical_min,
            physical_max,
            digital_min,
            digital_max,
            prefiltering,
            nr_samples,
            reserved,
        ]
        signal_headers[i] = ensemble_eeg.SignalHeader(*sig_header)

    return tuple(signal_headers)


def write_brm_data_to_edf(filename, data):
    """
    Write BRM data to EDF file.

    Args:
        filename (str): The name of the file to write the data to.
        data (list): A list of data objects.

    Returns:
        None
    """
    file_exists = os.path.isfile(filename)
    dat1 = data[0].data
    dat2 = data[1].data

    if file_exists:
        fs = data[0].sampleHz
        n_records = int(np.floor(len(data[0].data) / fs))
        fd = open(filename, "ab")

        index = 0
        for i_record in range(0, n_records):
            fd.write(dat1[index : (index + fs)])
            fd.write(dat2[index : (index + fs)])
            index = (i_record + 1) * fs

        fd.close()



#!/usr/bin/env python3
# setup.py
import setuptools

setuptools.setup(
    name="ensemble_edf",
    py_modules=["ensemble_edf", "brm_to_edf"],
    python_requires=">3.10",
    install_requires=["defusedxml", "numpy"],
    version="0.1.0",
    author="Bauke van der Velde",
    author_email="b.vandervelde-3@umcutrecht.nl",
)
